Package: iCheck
Type: Package
Title: QC Pipeline and Data Analysis Tools for High-Dimensional
        Illumina mRNA Expression Data
Version: 1.40.0
Date: 2016-11-17
Authors@R: c(person("Weiliang", "Qiu", role=c("aut", "cre"),
                     email = "stwxq@channing.harvard.edu"),
             person("Brandon", "Guo", role=c("aut", "ctb"),
                     email = "brandowonder@gmail.com"),
             person("Christopher", "Anderson", role = c("aut", "ctb"),
                     email = "christopheranderson84@gmail.com"),
             person("Barbara", "Klanderman", role = c("aut", "ctb"),
                     email = "BKLANDERMAN@partners.org"),
             person("Vincent", "Carey", role = c("aut", "ctb"),
                     email = "stvjc@channing.harvard.edu"),
             person("Benjamin", "Raby", role = c("aut", "ctb"),
                     email = "rebar@channing.harvard.edu")
            )
Author: Weiliang Qiu [aut, cre],
        Brandon Guo [aut, ctb],
        Christopher Anderson [aut, ctb],
        Barbara Klanderman [aut, ctb],
        Vincent Carey [aut, ctb],
        Benjamin Raby [aut, ctb]
Maintainer: Weiliang Qiu <stwxq@channing.harvard.edu>
Depends: R (>= 3.2.0), Biobase, lumi, gplots
Imports: stats, graphics, preprocessCore, grDevices, randomForest,
        affy, limma, parallel, GeneSelectMMD, rgl, MASS, lmtest,
        scatterplot3d, utils
Suggests:
biocViews: GeneExpression, DifferentialExpression, Microarray,
        Preprocessing, DNAMethylation, OneChannel, TwoChannel,
        QualityControl
Description: QC pipeline and data analysis tools for high-dimensional
        Illumina mRNA expression data.
License: GPL (>= 2)
NeedsCompilation: no
Packaged: 2025-10-30 04:32:10 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/iCheck
git_branch: RELEASE_3_22
git_last_commit: 109cb15
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
